import { AngularFireAuth } from '@angular/fire/auth';
import { AngularFirestore } from '@angular/fire/firestore';
import firebase from 'firebase/app';
import { Observable } from 'rxjs';
import { Store } from '@datorama/akita';
import { FireAuthState } from './auth.model';
import { WriteOptions, UpdateCallback } from '../utils/types';
export declare const authProviders: readonly ["github", "google", "microsoft", "facebook", "twitter", "email", "apple"];
export declare type FireProvider = (typeof authProviders)[number];
declare type UserCredential = firebase.auth.UserCredential;
declare type AuthProvider = firebase.auth.AuthProvider;
/** Verify if provider is part of the list of Authentication provider provided by Firebase Auth */
export declare function isFireAuthProvider(provider: any): provider is FireProvider;
/**
 * Get the custom claims of a user. If no key is provided, return the whole claims object
 * @param user The user object returned by Firebase Auth
 * @param roles Keys of the custom claims inside the claim objet
 */
export declare function getCustomClaims(user: firebase.User, roles?: string | string[]): Promise<Record<string, any>>;
/**
 * Get the Authentication Provider based on its name
 * @param provider string literal representing the name of the provider
 */
export declare function getAuthProvider(provider: FireProvider): firebase.auth.EmailAuthProvider | firebase.auth.FacebookAuthProvider | firebase.auth.GithubAuthProvider | firebase.auth.GoogleAuthProvider | firebase.auth.OAuthProvider | firebase.auth.TwitterAuthProvider;
export declare class FireAuthService<S extends FireAuthState> {
    protected store: Store<S>;
    private collection;
    protected collectionPath: string;
    protected db: AngularFirestore;
    auth: AngularFireAuth;
    /** Triggered when the profile has been created */
    protected onCreate?(profile: S['profile'], options: WriteOptions): any;
    /** Triggered when the profile has been updated */
    protected onUpdate?(profile: S['profile'], options: WriteOptions): any;
    /** Triggered when the profile has been deleted */
    protected onDelete?(options: WriteOptions): any;
    /** Triggered when user signin for the first time or signup with email & password */
    protected onSignup?(user: UserCredential, options: WriteOptions): any;
    /** Triggered when a user signin, except for the first time @see onSignup */
    protected onSignin?(user: UserCredential): any;
    /** Triggered when a user signout */
    protected onSignout?(): any;
    constructor(store: Store<S>, db?: AngularFirestore, auth?: AngularFireAuth);
    /**
     * Select the profile in the Firestore
     * @note can be override to point to a different place
     */
    protected selectProfile(user: firebase.User): Observable<S['profile']>;
    /**
     * Select the roles for this user. Can be in custom claims or in a Firestore collection
     * @param user The user given by FireAuth
     * @see getCustomClaims to get the custom claims out of the user
     * @note Can be overwritten
     */
    protected selectRoles(user: firebase.User): Promise<S['roles']> | Observable<S['roles']>;
    /**
     * Function triggered when getting data from firestore
     * @note should be overwritten
     */
    protected formatFromFirestore(user: any): S['profile'];
    /**
     * Function triggered when adding/updating data to firestore
     * @note should be overwritten
     */
    protected formatToFirestore(user: S['profile']): any;
    /**
     * Function triggered when transforming a user into a profile
     * @param user The user object from FireAuth
     * @param ctx The context given on signup
     * @note Should be override
     */
    protected createProfile(user: firebase.User, ctx?: any): Promise<Partial<S['profile']>> | Partial<S['profile']>;
    /**
     * The current sign-in user (or null)
     * @returns a Promise in v6.*.* & a snapshot in v5.*.*
     */
    get user(): Promise<firebase.User>;
    get idKey(): any;
    /** The path to the profile in firestore */
    get path(): any;
    /** Start listening on User */
    sync(): Observable<any[]>;
    /**
     * @description Delete user from authentication service and database
     * WARNING This is security sensitive operation
     */
    delete(options?: WriteOptions): Promise<void>;
    /** Update the current profile of the authenticated user */
    update(profile: Partial<S['profile']> | UpdateCallback<S['profile']>, options?: WriteOptions): Promise<void | firebase.firestore.Transaction>;
    /** Create a user based on email and password */
    signup(email: string, password: string, options?: WriteOptions): Promise<UserCredential>;
    /** Signin with email & password, provider name, provider objet or custom token */
    signin(email: string, password: string, options?: WriteOptions): Promise<UserCredential>;
    signin(authProvider: AuthProvider, options?: WriteOptions): Promise<UserCredential>;
    signin(provider?: FireProvider, options?: WriteOptions): Promise<UserCredential>;
    signin(token: string, options?: WriteOptions): Promise<UserCredential>;
    /** Signs out the current user and clear the store */
    signOut(): Promise<void>;
}
export {};
//# sourceMappingURL=auth.service.d.ts.map